﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.InteropServices;
using System.Diagnostics;
using System.IO;

namespace PH_FireTV
{
    public interface IPHFireTV
    {
        Int16 PHPI_Init(Int32 hPH, Int16 iI, ref string sID);
        double PHPI_SDKVersion();
        string PHPI_GetPIVersion();
        string PHPI_GetPIInfo();
        Int16 PHPI_Enable();
        Int16 PHPI_Disable();
        Int16 PHPI_Status();
        Int16 PHPI_InitData(ref string sData);
        Int16 PHPI_Close();
        string PHPI_Command(Int32 lCmd, Int32 lData1, Int32 lData2, ref string sData1, ref string sData2);
        string PHPI_SysCommand(Int32 lCmd, Int32 lData1, Int32 lData2, ref string sData1, ref string sData2);
        string PHPI_WebCmd(ref string sAction, Int16 iAuth, ref string sB64Auth, ref string sIP, ref string sParms);
    }
    public class phfiretv : IPHFireTV
    {
        [DllImport("kernel32.dll", EntryPoint = "GetPrivateProfileString")]
        private static extern int GetPrivateProfileString(string lpAppName, string lpKeyName, string lpDefault, StringBuilder lpReturnedString, int nSize, string lpFileName);

        Int32 hPwrHome;
        Int16 iIndex;
        string sPHID;
        Int16 iStatus;
        PHConstants.COPYDATASTRUCT cds;

        string sINIpath;
        string sIP;
        string sADBpath;
        Boolean bKill;

        Process process;
        StreamWriter sw;

        public Int16 PHPI_Init(Int32 hPH, Int16 iI, ref string sID)
        {
            hPwrHome = hPH;
            iIndex = iI;
            sPHID = sID;
            iStatus = 0;

            return 0;
        }

        public double PHPI_SDKVersion()
        {
            return 1.2;
        }

        public string PHPI_GetPIVersion()
        {
            return "1.2";
        }

        public string PHPI_GetPIInfo()
        {
            return "PowerHome FireTV ADB plugin";
        }

        public Int16 PHPI_Enable()
        {
            iStatus = 1;

            process = new System.Diagnostics.Process();
            process.StartInfo.FileName = "cmd.exe";
            process.StartInfo.WorkingDirectory = sADBpath;
            process.StartInfo.WindowStyle = System.Diagnostics.ProcessWindowStyle.Normal;
            process.StartInfo.RedirectStandardInput = true;
            process.StartInfo.RedirectStandardOutput = true;
            process.StartInfo.UseShellExecute = false;
            process.StartInfo.CreateNoWindow = true;
            process.Start();
            sw = process.StandardInput;

            if (bKill) sw.WriteLine("adb kill-server");
            sw.WriteLine("adb start-server");
            sw.WriteLine("adb connect " + sIP);
            sw.WriteLine("adb -s " + sIP + ":5555 shell");

            return 0;
        }

        public Int16 PHPI_Disable()
        {
            if (iStatus == 0)
                return 0;

            iStatus = 0;

            sw.WriteLine("adb disconnect " + sIP);
            if (bKill) sw.WriteLine("adb kill-server");
            sw.WriteLine("exit");
            sw.Close();
            process.Kill();
            process.Close();
            process.Dispose();

            if (bKill)
            {
                process = new System.Diagnostics.Process();
                process.StartInfo.FileName = "cmd.exe";
                process.StartInfo.WorkingDirectory = sADBpath;
                process.StartInfo.Arguments = "/k \"adb disconnect " + sIP + "&&adb kill-server\"";
                process.StartInfo.WindowStyle = System.Diagnostics.ProcessWindowStyle.Normal;
                process.StartInfo.RedirectStandardInput = true;
                process.StartInfo.RedirectStandardOutput = true;
                process.StartInfo.UseShellExecute = false;
                process.StartInfo.CreateNoWindow = true;
                process.Start();
                System.Threading.Thread.Sleep(200);

                process.Kill();
                process.Close();
                process.Dispose();
            }

            return 0;
        }

        public Int16 PHPI_Status()
        {
            return iStatus;
        }

        public Int16 PHPI_InitData(ref string sData)
        {
            int chars = 256;
            StringBuilder buffer = new StringBuilder(chars);
            string sDefault = "";
            string sTemp;

            sINIpath = sData.Trim();

            if (GetPrivateProfileString("settings", "ip", sDefault, buffer, chars, sINIpath) != 0)
                sIP = buffer.ToString();

            if (GetPrivateProfileString("settings", "adbpath", sDefault, buffer, chars, sINIpath) != 0)
                sADBpath = buffer.ToString();

            if (GetPrivateProfileString("settings", "kill", sDefault, buffer, chars, sINIpath) != 0)
            {
                sTemp = buffer.ToString().ToLower();
                if (sTemp.CompareTo("yes") == 0 || sTemp.CompareTo("1") == 0)
                    bKill = true;
                else
                    bKill = false;
            }

            return 0;
        }

        public Int16 PHPI_Close()
        {
            return 0;
        }

        public string PHPI_Command(Int32 lCmd, Int32 lData1, Int32 lData2, ref string sData1, ref string sData2)
        {
            int iChar;

            if (iStatus != 1) return "*ERROR* - Disabled";

            sData1 = sData1.ToLower();

            switch (sData1)
            {
                case "killserver":
                    if (lData1 == 0) bKill = false; else bKill = true;
                    break;
                case "up":
                    sw.WriteLine("input keyevent 19");
                    break;
                case "down":
                    sw.WriteLine("input keyevent 20");
                    break;
                case "left":
                    sw.WriteLine("input keyevent 21");
                    break;
                case "right":
                    sw.WriteLine("input keyevent 22");
                    break;
                case "enter":
                case "select":
                    sw.WriteLine("input keyevent 66");
                    break;
                case "back":
                    sw.WriteLine("input keyevent 4");
                    break;
                case "home":
                    sw.WriteLine("input keyevent 3");
                    break;
                case "menu":
                    sw.WriteLine("input keyevent 82");
                    break;
                case "play":
                case "pause":
                    sw.WriteLine("input keyevent 85");
                    break;
                case "prev":
                    sw.WriteLine("input keyevent 88");
                    break;
                case "next":
                    sw.WriteLine("input keyevent 87");
                    break;
                case "power":
                    sw.WriteLine("input keyevent 26");
                    break;
                case "volup":
                    sw.WriteLine("input keyevent 24");
                    break;
                case "voldn":
                    sw.WriteLine("input keyevent 25");
                    break;
                case "delete":
                    sw.WriteLine("input keyevent 67");
                    break;
                case "tab":
                    sw.WriteLine("input keyevent 61");
                    break;
                case "key":
                    sData2 = sData2.ToLower();
                    for (int x = 0; x < sData2.Length; x++)
                    {
                        iChar = (int)sData2.Substring(x, 1)[0];
                        if (iChar >= 97 && iChar <= 122)
                            iChar -= 68;
                        else if (iChar >= 48 && iChar <= 57)
                            iChar -= 41;
                        else if (iChar >= 40 && iChar <= 41)
                            iChar += 31;
                        else if (iChar == 42)
                            iChar = 17;
                        else if (iChar == 35)
                            iChar = 18;
                        else if (iChar == 44)
                            iChar = 55;
                        else if (iChar == 46)
                            iChar = 56;
                        else if (iChar == 9)
                            iChar = 61;
                        else if (iChar == 32)
                            iChar = 62;
                        else if (iChar == 45)
                            iChar = 69;
                        else if (iChar == 61)
                            iChar = 70;
                        else if (iChar == 92)
                            iChar = 73;
                        else if (iChar == 59)
                            iChar = 74;
                        else if (iChar == 39)
                            iChar = 75;
                        else if (iChar == 47)
                            iChar = 76;
                        else if (iChar == 64)
                            iChar = 77;
                        else if (iChar == 43)
                            iChar = 81;
                        else
                            continue;

                        sw.WriteLine("input keyevent " + iChar.ToString());
                    }
                    break;
                default:
                    return "*ERROR* - Invalid key";
            }

            return "SUCCESS";
        }

        public string PHPI_SysCommand(Int32 lCmd, Int32 lData1, Int32 lData2, ref string sData1, ref string sData2)
        {
            return "";
        }

        public string PHPI_WebCmd(ref string sAction, Int16 iAuth, ref string sB64Auth, ref string sIP, ref string sParms)
        {
            return "<html><head><title>Not Supported</title></head><body><h2>No supported web pages for PowerHome Plugin: " + sPHID + "</h2></body></html>";
        }

        private string ph_formula(ref string sFormula, Int32 lType)
        {
            //lType = 1000 - Use current PH variable set and wait for return
            //lType = 1001 - Create new PH variable set and wait for return
            //lType = 1002 - Use current PH variable set and return immediately
            //lType = 1003 - Create new PH variable set and return immediately

            int lRet;

            cds.dwData = lType;
            cds.cbData = sFormula.Length;
            cds.lpData = sFormula;

            lRet = PHConstants.SendMessage(hPwrHome, 74, iIndex, ref cds);

            if (lRet != 0)
            {
                cds = (PHConstants.COPYDATASTRUCT)Marshal.PtrToStructure((IntPtr)lRet, typeof(PHConstants.COPYDATASTRUCT));
            }

            return cds.lpData;
        }

        private string f_fixquotes(string sData)
        {
            sData = sData.Replace("'", "~h7E~h27");
            return sData;
        }
    }

    internal class PHConstants
    {
        public struct COPYDATASTRUCT
        {
            public int dwData;
            public int cbData;
            [MarshalAs(UnmanagedType.LPStr)]
            public string lpData;
        }

        [DllImport("user32.dll")]
        public static extern int SendMessage(int hwnd, int msg, int wParam, ref COPYDATASTRUCT lParam);

        [DllImport("Kernel32.dll")]
        public static extern void RtlMoveMemory(ref int dest, int src, int size);

        [DllImport("Kernel32.dll")]
        public static extern void RtlMoveMemory(ref string dest, int src, int size);

        [DllImport("User32.dll")]
        public static extern int MessageBox(int h, string m, string c, int type);

        public enum MessageBoxResult : uint
        {
            Ok = 1,
            Cancel,
            Abort,
            Retry,
            Ignore,
            Yes,
            No,
            Close,
            Help,
            TryAgain,
            Continue,
            Timeout = 32000
        }
    }
}
